import datetime
import mock
import unittest

from lpreports.launchpad import criterias
from test_filters import Bug

DUMMY_MAJOUR_TEAMS = {
    u'MOS OpenStack': [
        u'mos-neutron',
        u'mos-nova',
    ],
    u'Fuel': [
        u'vkuklin',
    ],
    u'Customer-Enablement': [],
    u'Workloads': [],
    u'Maintenance': [],
    u'L23Networking': [],
    u'Plugins': [],
    u'Docs': [],
    u'PCE': [],
    u'Infra': [],
    u'QA': [],
    u'Unknown': {u'unknown': []},
}


class CriteriasTestCase(unittest.TestCase):
    def setUp(self):
        self.app = self._create_app()

    @mock.patch('lpreports.db.init_app')
    def _create_app(self, init_db):
        init_db.return_value = None
        from lpreports import application
        return application.create_app(test_mode=True)

    def _create_triage_criteria(self, **kwargs):
        owners = {
            "mos": "MOS OpenStack",
            "fuel": "Fuel"
        }
        criterias_kwargs = dict(
            incomplete_threshold=5,
            teams_involved=["MOS OpenStack", "Fuel"],
            project_owners=owners,
            queue_owner="MOS OpenStack",
            included_tags=[],
            involved_tags=[],
            default_queue=False
        )
        criterias_kwargs.update(kwargs)
        return criterias.NonTriaged(**criterias_kwargs)

    def test_calendar_days_ago(self):
        pass

    def test_business_days_ago(self):
        test_cases = (
            # current date, days_ago, expected result
            # return current date if days == 0
            (datetime.datetime(2016, 6, 19), 0, datetime.datetime(2016, 6, 19)),
            # in case if current day is weekend
            (datetime.datetime(2016, 6, 19), 1, datetime.datetime(2016, 6, 17)),
            (datetime.datetime(2016, 6, 18), 1, datetime.datetime(2016, 6, 17)),
            # in case if target day is weekend
            (datetime.datetime(2016, 6, 16), 4, datetime.datetime(2016, 6, 10)),
            (datetime.datetime(2016, 6, 16), 5, datetime.datetime(2016, 6, 9)),
            # if current day and target day are weekdays
            (datetime.datetime(2016, 6, 16), 6, datetime.datetime(2016, 6, 8)),
            # if current day and target day are weekends
            (datetime.datetime(2016, 6, 18), 6, datetime.datetime(2016, 6, 10)),
            # if days param is very big value
            (datetime.datetime(2016, 6, 18), 10000, datetime.datetime(1978, 2, 20, 0, 0)),
        )
        fake_datetime = mock.Mock()
        for now, days, expected_result in test_cases:
            with mock.patch('lpreports.launchpad.criterias.datetime.datetime', fake_datetime):
                fake_datetime.now.return_value = now
                self.assertEqual(criterias.business_days_ago(days), expected_result)

    def test_business_days_calc(self):
        pass

    @mock.patch('lpreports.db.get_major_teams')
    def test_triage_by_incomplete_date(self, get_major_teams):
        get_major_teams.return_value = DUMMY_MAJOUR_TEAMS
        with self.app.app_context():
            bug = Bug(
                assignee="mos-neutron",
                status="Incomplete",
                date_incomplete=criterias.calendar_days_ago(6)
            )

            triage = self._create_triage_criteria()
            self.assertTrue(triage.apply(bug))

            bug.date_incomplete = criterias.calendar_days_ago(1)
            self.assertFalse(triage.apply(bug))

    @mock.patch('lpreports.db.get_major_teams')
    def test_triage_by_target_project(self, get_major_teams):
        get_major_teams.return_value = DUMMY_MAJOUR_TEAMS
        with self.app.app_context():
            bug = Bug(
                status="New",
                target_name="mos",
                assignee=None
            )

            triage = self._create_triage_criteria()
            self.assertTrue(triage.apply(bug))

            # Bug should be triaged by other team (Fuel team)
            bug.assignee = "vkuklin"
            self.assertFalse(triage.apply(bug))

    @mock.patch('lpreports.db.get_major_teams')
    def test_triage_by_milestone(self, get_major_teams):
        get_major_teams.return_value = DUMMY_MAJOUR_TEAMS
        with self.app.app_context():
            bug = Bug(
                milestone=None,
                target_name="mos",
                importance="Critical",
                assignee="someone"
            )
            triage = self._create_triage_criteria()

            self.assertTrue(triage.apply(bug))

            bug.milestone = "1.0"
            self.assertFalse(triage.apply(bug))

    @mock.patch('lpreports.db.get_major_teams')
    def test_triage_by_importance(self, get_major_teams):
        get_major_teams.return_value = DUMMY_MAJOUR_TEAMS
        with self.app.app_context():
            bug = Bug(
                milestone="1.0",
                assignee="mos-nova",
                importance="Undecided",
            )
            triage = self._create_triage_criteria()

            self.assertTrue(triage.apply(bug))

            bug.importance = "High"
            self.assertFalse(triage.apply(bug))

    @mock.patch('lpreports.db.get_major_teams')
    def test_triage_by_tags(self, get_major_teams):
        get_major_teams.return_value = DUMMY_MAJOUR_TEAMS
        with self.app.app_context():

            # case 1: bug contains some included_tags and
            # doesn't contain any of involved_tags
            bug = Bug(
                status='New',
                tags=['tag1', 'tag2']
            )
            triage = self._create_triage_criteria(
                included_tags=['tag1'],
                involved_tags=['tag0', 'tag1'],
                default_queue=True
            )
            self.assertTrue(triage.apply(bug))

            # case 2: bug contains some included_tags and some involved_tags
            # when queue is default
            triage = self._create_triage_criteria(
                included_tags=['tag1'],
                involved_tags=['tag1', 'tag2'],
                default_queue=True
            )
            self.assertFalse(triage.apply(bug))

            # case 3: bug contains some included_tags and some involved_tags
            # when queue is not default
            triage = self._create_triage_criteria(
                included_tags=['tag1'],
                involved_tags=['tag1', 'tag2'],
                default_queue=False
            )
            self.assertTrue(triage.apply(bug))
