from collections import defaultdict
import datetime
import mock
import unittest

from lpreports.launchpad import filters


# FIXME(ruhe): move this class out of this file. It's being used outside too
class Bug(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.title = kwargs.get('title', None)
        self.status = kwargs.get('status', None)
        self.importance = kwargs.get('importance', None)
        self.assignee = kwargs.get('assignee', None)
        self.assignee_link = kwargs.get('assignee_link', None)
        self.target = kwargs.get('target', None)
        self.milestone = kwargs.get('milestone', None)
        self.original_milestone = kwargs.get('original_milestone', None)
        self.tags = kwargs.get('tags', [])
        self.target_name = kwargs.get('target_name', '')
        self.weight = kwargs.get('weight', '')
        self.date_created = kwargs.get('date_created', None)
        self.date_triaged = kwargs.get('date_triaged', None)
        self.date_fix_committed = kwargs.get('date_fix_committed', None)
        self.date_fix_released = kwargs.get('date_fix_released', None)
        self.date_incomplete = kwargs.get('date_incomplete', None)
        self.date_last_updated = kwargs.get('date_last_updated', None)

    def __repr__(self):
        return "#{0} '{1}'".format(self.id, self.title)


class FiltersTestCase(unittest.TestCase):

    @mock.patch('lpreports.db.init_app')
    def _create_app(self, init_db):
        init_db.return_value = None
        from lpreports import application
        return application.create_app(test_mode=True)

    def create_filter_params(self, **kwargs):
        params = defaultdict(lambda: [])
        params['created_to'] = None
        params['created_from'] = None
        params['triaged_to'] = None
        params['triaged_from'] = None
        params['fix_committed_to'] = None
        params['fix_committed_from'] = None
        params['fix_released_to'] = None
        params['fix_released_from'] = None

        params.update(**kwargs)

        return params

    def full_teams(self):
        return {
            "Fuel": {
                "fuel-build": ["fuel-build#1", "fuel-build#2", "fuel-build"],
                "fuel-core": ["fuel-core#1", "fuel-core#2", "fuel-core"],
                "fuel-qa": ["super-qa#1"]
            },
            "MOS": {
                "mos-nova": ["mos-nova", "nova#1", "nova#2"]
            }
        }

    def _wrap_bug_data(self, bugs):
        return {
            "DATA": [{"bugs": bugs}]
        }

    def _unwrap_bug_data(self, data):
        return data["DATA"][0]["bugs"]

    def filter_bugs(self, bugs, _filters, teams_data):
        data = self._wrap_bug_data(bugs)
        output = filters.filter_bugs(data, _filters, teams_data)
        return self._unwrap_bug_data(output)

    def assert_bugs_by_id(self, expected_ids, bugs):
        found_ids = map(lambda x: x.id, bugs)
        self.assertListEqual(expected_ids, found_ids)

    def test_filter_by_unknown_assignee(self):
        bugs = [
            Bug(id=1, assignee=None),
            Bug(id=2, assignee="fuel-qa"),
            Bug(id=3, assignee="fuel-core#1"),
            Bug(id=4, assignee="somebody"),
        ]

        f = self.create_filter_params(assignee='unknown')
        found = self.filter_bugs(bugs, f, self.full_teams())
        self.assert_bugs_by_id([1, 4], found)

    def test_filter_by_assignee(self):
        bugs = [
            Bug(id=1, assignee='nova#1'),
            Bug(id=2, assignee='fuel-core#1'),
            Bug(id=3, assignee='mos-nova')
        ]

        params = self.create_filter_params(assignee='mos-nova')
        found = self.filter_bugs(bugs, params, self.full_teams())
        self.assert_bugs_by_id([1, 3], found)

    def test_filter_by_assigned_person(self):
        bugs = [
            Bug(id=1, assignee='nova#1'),
            Bug(id=2, assignee='fuel-core#1'),
            Bug(id=3, assignee='mos-nova')
        ]

        params = self.create_filter_params(assigned_person='mos-nova')
        found = self.filter_bugs(bugs, params, self.full_teams())
        self.assert_bugs_by_id([3], found)

    def test_filter_by_importance(self):
        bugs = [
            Bug(id=1, importance='Critical'),
            Bug(id=2, importance='High'),
            Bug(id=3, importance='Medium'),
            Bug(id=4, importance='Low'),
            Bug(id=5, importance='High')
        ]

        params = self.create_filter_params(importance=['High', 'Medium'])
        found = self.filter_bugs(bugs, params, self.full_teams())
        self.assert_bugs_by_id([2, 3, 5], found)

    def test_filter_by_tags(self):
        bugs = [
            Bug(id=1, tags=['a', 'b', 'c']),
            Bug(id=2, tags=['x', 'y', 'z']),
            Bug(id=3, tags=['a', 'b', 'x']),
            Bug(id=4, tags=['a', 'y', 'z']),
        ]

        params = self.create_filter_params(tags=['x', 'y'])
        found = self.filter_bugs(bugs, params, self.full_teams())
        self.assert_bugs_by_id([2, 3, 4], found)

    def test_filter_by_tags_all(self):
        bugs = [
            Bug(id=1, tags=['a', 'b', 'c']),
            Bug(id=2, tags=['x', 'y', 'z']),
            Bug(id=3, tags=['a', 'b', 'x']),
            Bug(id=4, tags=['a', 'y', 'z']),
        ]

        params = self.create_filter_params(tags_all=['x', 'y'])
        found = self.filter_bugs(bugs, params, self.full_teams())
        self.assert_bugs_by_id([2], found)

    def test_filter_by_excluded_tags(self):
        bugs = [
            Bug(id=1, tags=['a', 'b', 'c']),
            Bug(id=2, tags=['x', 'y', 'z']),
            Bug(id=3, tags=['a', 'b', 'z']),
            Bug(id=4, tags=['a', 'y', 'z']),
        ]

        params = self.create_filter_params(exclude_tags=['c', 'y'])
        found = self.filter_bugs(bugs, params, self.full_teams())
        self.assert_bugs_by_id([3], found)

    def _get_date(self, date_str):
        return datetime.datetime.strptime(date_str, "%Y-%m-%d")

    def test_filter_by_dates(self):
        bugs = [
            Bug(id=1, date_created=self._get_date("2012-02-23")),
            Bug(id=2, date_created=self._get_date("2012-01-25")),
            Bug(id=3, date_created=self._get_date("2012-03-08")),
            Bug(id=4, date_created=self._get_date("2012-05-01"))
        ]

        params = self.create_filter_params(created_from="2012-02-01",
                                           created_to="2012-04-30")
        found = self.filter_bugs(bugs, params, self.full_teams())
        self.assert_bugs_by_id([1, 3], found)

    def test_filter_by_only_user_affecting(self):
        bugs = [
            Bug(id=1, tags=['feature', 'a', 'b']),
            Bug(id=2, tags=['enhancement', 'c']),
            Bug(id=3, tags=['d', 'e', 'f']),
            Bug(id=4, tags=['area-ci'], importance='Low'),
            Bug(id=5, tags=['area-ci'], importance='High'),
            Bug(id=6, tags=['area-ci', 'feature'], importance='High'),
        ]

        params = self.create_filter_params(only_user_affecting=True)
        with self._create_app().app_context():
            found = self.filter_bugs(bugs, params, self.full_teams())
        self.assert_bugs_by_id([3, 5], found)

    def test_filter_by_original_milestone(self):
        bugs = [
            # origin milestone: 7.0
            # shouldn't be found by the filter
            Bug(id=1, milestone='7.0-updates'),
            Bug(id=1, milestone='8.0'),
            Bug(id=1, milestone='8.1'),

            # origin milestone: 8.0
            Bug(id=2, milestone='8.0'),
            Bug(id=2, milestone='8.1'),

            # origin milestone: 8.0.1
            # shouldn't be found by the filter
            Bug(id=3, milestone='8.0.1-mu'),
            Bug(id=3, milestone='8.1'),

            # origin milestone: 9.0
            Bug(id=4, milestone='9.0-updates'),
            Bug(id=4, milestone='10.0'),
        ]

        test_cases = (
            # original_milestones, expected result
            (['8.0'], [2]),  # usual case
            (['8.0', '9.0'], [2, 4]),  # in case if user specify few origin milestones
            (['9.0-mu'], [4]),  # check if suffix is cut properly
        )
        for original_milestones, expected_result in test_cases:
            params = self.create_filter_params(original_milestone=original_milestones)
            found = self.filter_bugs(bugs, params, self.full_teams())
            self.assert_bugs_by_id(expected_result, found)

    def test_multiple_filters(self):
        bugs = [
            Bug(id=1,
                milestone='6.1',
                status='Confirmed',
                importance='Critical',
                tags=['heat', 'scale'],
                date_created=self._get_date("2012-02-01"),
                date_triaged=self._get_date("2012-02-03")),
            Bug(id=2,
                milestone='6.1',
                status='In Progress',
                importance='High',
                tags=['neutron', 'scale'],
                date_created=self._get_date("2012-03-05"),
                date_triaged=self._get_date("2012-03-05")),
            Bug(id=3,
                milestone='7.0',
                status='Confirmed',
                importance='Medium',
                tags=['heat'],
                date_created=self._get_date("2012-02-10"),
                date_triaged=self._get_date("2012-02-15"))
        ]

        params = self.create_filter_params(milestone='6.1',
                                           importance=['High', 'Medium'],
                                           tags=['scale'],
                                           created_from="2012-03-01")
        found = self.filter_bugs(bugs, params, self.full_teams())
        self.assert_bugs_by_id([2], found)
