import mock
import unittest

from lpreports.launchpad import sla_reports

BUGS_DB_FUEL = [
    {
        u'id': 1,
        u'title': u'Fuel test bug #1',
        u'target_name': u'fuel',
        u'private': True,
        u'assignee': u'vpupkin',
        u'owner': u'pivanov',
        u'tags': [],
        u'status': u'New',
        u'importance': u'High',
        u'information_type': u'Private Security'
    },
    {
        u'id': 2,
        u'title': u'Fuel test bug #2',
        u'target_name': u'fuel',
        u'private': False,
        u'assignee': u'vpupkin',
        u'owner': u'pivanov',
        u'tags': [],
        u'status': u'Incomplete',
        u'importance': u'High',
        u'information_type': u'Public'
    },
    {
        u'id': 3,
        u'title': u'Fuel test bug #3',
        u'target_name': u'fuel',
        u'private': False,
        u'assignee': u'vpupkin',
        u'owner': u'pivanov',
        u'tags': [],
        u'status': u'Incomplete',
        u'importance': u'Critical',
        u'information_type': u'Public'
    },
    {
        u'id': 4,
        u'title': u'Fuel test bug #4',
        u'target_name': u'fuel',
        u'private': False,
        u'assignee': u'vpupkin',
        u'owner': u'pivanov',
        u'tags': [u'customer-found'],
        u'status': u'Incomplete',
        u'importance': u'Critical',
        u'information_type': u'Public'
    }
]

BUGS_DB_MOS = [
    {
        u'id': 11,
        u'title': u'MOS test bug #11',
        u'target_name': u'mos',
        u'private': True,
        u'assignee': u'vpupkin',
        u'owner': u'pivanov',
        u'tags': [],
        u'status': u'New',
        u'importance': u'High',
        u'information_type': u'Private Security'
    },
    {
        u'id': 12,
        u'title': u'MOS test bug #12',
        u'target_name': u'mos',
        u'private': False,
        u'assignee': u'vpupkin',
        u'owner': u'pivanov',
        u'tags': [],
        u'status': u'Incomplete',
        u'importance': u'High',
        u'information_type': u'Public'
    },
    {
        u'id': 13,
        u'title': u'MOS test bug #13',
        u'target_name': u'mos',
        u'private': False,
        u'assignee': u'vpupkin',
        u'owner': u'pivanov',
        u'tags': [],
        u'status': u'Incomplete',
        u'importance': u'Critical',
        u'information_type': u'Public'
    },
    {
        u'id': 14,
        u'title': u'MOS test bug #14',
        u'target_name': u'mos',
        u'private': False,
        u'assignee': u'vpupkin',
        u'owner': u'pivanov',
        u'tags': [u'customer-found'],
        u'status': u'Incomplete',
        u'importance': u'Critical',
        u'information_type': u'Public'
    }
]


class SlaReportTestCase(unittest.TestCase):

    def assertSequenceStartWith(self, original, expected):
        for o_item, e_item in zip(original, expected):
            if o_item != e_item:
                raise AssertionError("%r doesn't start with %r" % (original, expected))

    @mock.patch('lpreports.launchpad.sla_reports.get_criteria_by_name')
    @mock.patch('lpreports.db.bugs_db')
    def test_get_bugs_by_criteria(self, bugs_db, get_criteria_by_name):
        # mock bugs_db
        fuel_stage = mock.Mock()
        fuel_find_mock = mock.Mock(return_value=BUGS_DB_FUEL)
        fuel_stage.find = fuel_find_mock

        mos_stage = mock.Mock()
        mos_find_mock = mock.Mock(return_value=BUGS_DB_MOS)
        mos_stage.find = mos_find_mock

        bugs_db.return_value = dict(fuel=fuel_stage, mos=mos_stage)

        # mock get_criteria_by_name
        get_criteria_by_name.return_value = {
            'short-text': 'all',
            'text': 'Criteria including all the bugs',
            'name': 'all',
            'implementation': 'criterias.All'
        }

        # default args to call the function
        kwargs = {
            'criterias': [
                {'name': 'all'}
            ],
            'projects': ['fuel', 'mos'],
            'milestone_names': ['10.0', '9.0'],
            'options': {
                'status': ['New', 'Incomplete'],
                'importance': ['High', 'Critical']
            },
            'security_membership': ['fuel']
        }

        # default expected DB query
        expected_db_query = {
            '$and': [
                {
                    'status': {'$in': ['New', 'Incomplete']}
                },
                {
                    'importance': {'$in': ['High', 'Critical']}
                },
                {
                    'milestone': {'$in': ['10.0', '9.0']}
                },
                {
                    'is_duplicate': {'$ne': True}
                }
            ]
        }

        # Test case #1
        # When:
        #   skip_duplicates is True (default value)
        #   security_membership is ['fuel']

        result = sla_reports.get_bugs_by_criteria(**kwargs)

        # 1.1 check DB queries
        for obj in (fuel_find_mock, mos_find_mock):
            obj.assert_called_once_with(expected_db_query)
            obj.reset_mock()

        # 1.2 check bugs count
        self.assertEqual(len(result), 7)

        # 1.3 check bugs order
        self.assertSequenceStartWith(
            [b.id for b in result],
            [1, 2, 3, 4]
        )

        # Test case #2
        # When:
        #   skip_duplicates is False
        #   security_membership is ['fuel', 'mos']

        kwargs['skip_duplicates'] = False
        kwargs['security_membership'] = ['fuel', 'mos']

        result = sla_reports.get_bugs_by_criteria(**kwargs)

        # 2.1 check DB queries
        del expected_db_query['$and'][3]
        for obj in (fuel_find_mock, mos_find_mock):
            obj.assert_called_once_with(expected_db_query)
            obj.reset_mock()

        # 2.2 check bugs count
        self.assertEqual(len(result), 8)

        # 2.3 check bugs order
        self.assertSequenceStartWith(
            [b.id for b in result],
            [1, 2, 3, 4]
        )
